import React, { createContext, useState, useRef, useEffect, useCallback } from 'react';
import { getTodo, addTodo, delTodo, uptTodo } from '../../API/Api';
import { Header } from '../../Layout/card/Index';
import styles from './styles.module.css';
import List from '../../todo/List/Index';

export const MyTask = createContext();

export const Input = () => {
  const [task, setTask] = useState("");
  const [store, setStore] = useState([]);
  const [editIndex, setEditIndex] = useState(null);
  const [editedTask, setEditedTask] = useState("");
  const [loading, setLoading] = useState(false);
  const [isNewTodoAdded, setIsNewTodoAdded] = useState(false);
  const editableRef = useRef(null);
  const lastCursorPos = useRef(0);


  const debounceTimeout = useRef(null);

  const fetchTodos = useCallback(async () => {
    try {
      setLoading(true);
      const res = await getTodo();
      setStore(res.data || []);
    } catch (error) {
      console.error("Failed to fetch todos:", error);
    } finally {
      setLoading(false);
    }
  }, []);

  useEffect(() => {
    fetchTodos();
  }, [fetchTodos]);

  const saveCursorPosition = () => {
    try {
      if (editableRef.current) {
        const selection = window.getSelection();
        if (selection.rangeCount > 0) {
          const range = selection.getRangeAt(0);
          const preCaretRange = range.cloneRange();
          preCaretRange.selectNodeContents(editableRef.current);
          preCaretRange.setEnd(range.endContainer, range.endOffset);
          lastCursorPos.current = preCaretRange.toString().length;
        }
      }
    } catch (error) {
      console.error("Error saving cursor position:", error);
    }
  };

  const restoreCursorPosition = () => {
    try {
      if (editableRef.current) {
        const range = document.createRange();
        const selection = window.getSelection();

        let found = false;
        const findPosition = (node, remainingPos) => {
          if (node.nodeType === Node.TEXT_NODE) {
            const length = node.nodeValue.length;
            if (remainingPos <= length) {
              range.setStart(node, remainingPos);
              found = true;
              return;
            }
            remainingPos -= length;
          } else {
            for (let i = 0; i < node.childNodes.length && !found; i++) {
              findPosition(node.childNodes[i], remainingPos);
            }
          }
        };

        findPosition(editableRef.current, lastCursorPos.current);
        if (!found) {
          range.selectNodeContents(editableRef.current);
          range.collapse(false);
        } else {
          range.collapse(true);
        }

        selection.removeAllRanges();
        selection.addRange(range);
      }
    } catch (error) {
      console.error("Error restoring cursor position:", error);
    }
  };

  useEffect(() => {
    if (editableRef.current && editIndex !== null) {
      try {
        editableRef.current.focus();
        setTimeout(restoreCursorPosition, 0);
      } catch (error) {
        console.error("Error focusing editable element:", error);
        setEditIndex(null);
      }
    }
  }, [editIndex, editedTask]);

 
  const handleEdit = (e) => {
    try {
      saveCursorPosition();
      const newValue = e.currentTarget.textContent;
      setEditedTask(newValue);

      
      const updatedStore = [...store];
      const original = updatedStore[editIndex];
      updatedStore[editIndex] = { ...original, task: newValue };
      setStore(updatedStore);

     
      if (debounceTimeout.current) {
        clearTimeout(debounceTimeout.current);
      }

    
      debounceTimeout.current = setTimeout(() => {
        uptTodo(original.id, {
          id: original.id,
          task: newValue,
          completed: original.completed,
        }).catch((error) => {
          console.error("Error saving todo:", error);
        });
      }, 1000);
    } catch (error) {
      console.error("Error during edit:", error);
      setEditIndex(null);
    }
  };

  const handleChange = (e) => setTask(e.target.value);

  const startEdit = (index) => {
    try {
      setEditIndex(index);
      setEditedTask(store[index].task);
    } catch (error) {
      console.error("Error starting edit:", error);
      setEditIndex(null);
    }
  };

  const endEdit = () => {
    setEditIndex(null);
  };

  const Add = async () => {
    if (task.trim() === '') return;
    try {
      setIsNewTodoAdded(true);
      const res = await addTodo(task);
      setStore([...store, res.data]);
      setTask("");
    } catch (error) {
      console.error("Error adding todo:", error);
    } finally {
      setTimeout(() => setIsNewTodoAdded(false), 500);
    }
  };
  const Delete = async (id) => {
    const confirmDelete = window.confirm("Are you sure you want to delete?");
    if (confirmDelete) {
      try {
        await delTodo(id);
        setStore(store.filter(item => item.id !== id));
      } catch (error) {
        console.error("Error deleting todo:", error);
      }
    }
  };

  const toggleComplete = async (index) => {
    try {
      const newStore = [...store];
      newStore[index] = { 
        ...newStore[index], 
        completed: !newStore[index].completed 
      };
      setStore(newStore);

      await uptTodo(newStore[index].id, newStore[index]);
    } catch (error) {
      console.error("Error toggling complete:", error);
      fetchTodos();
    }
  };

  const renderEditableContent = () => {
    try {
      return (
        <div
          ref={editableRef}
          contentEditable
          suppressContentEditableWarning
          onBlur={endEdit}
          onInput={handleEdit}
          onKeyDown={(e) => {
            if (e.key === 'Enter') {
              e.preventDefault();
              e.currentTarget.blur();
            }
          }}
          className={`${styles.task} ${store[editIndex]?.completed ? 'completed' : ''}`}
          style={{
            display: 'inline-block',
            width:'100%',
            boxSizing:'border-box'
          }}
        >
          {editedTask}
        </div>
      );
    } catch (error) {
      console.error("Error rendering editable content:", error);
      return (
        <span 
          className={`${styles.task} ${store[editIndex]?.completed ? styles.completed : ''}`}
          onClick={() => startEdit(editIndex)}
        >
          {typeof store[editIndex]?.task === 'string'
            ? store[editIndex].task
            : JSON.stringify(store[editIndex]?.task)}
        </span>
      );
    }
  };

  return (
    <MyTask.Provider value={{ store, setStore }}>
      <div className="todo-container">
        <Header />
        <List
          store={store}
          editIndex={editIndex}
          renderEditableContent={renderEditableContent}
          startEdit={startEdit}
          toggleComplete={toggleComplete}
          Delete={Delete}
          isNewTodoAdded={isNewTodoAdded}
        />
        <form onSubmit={(e) => e.preventDefault()}>
          <div className={styles.merge}>
            <input
              type="text"
              className={styles.input}
              placeholder="Enter Items"
              value={task}
              onChange={handleChange}
              disabled={loading}
            />
            <button
              onClick={Add}
              className={styles.button}
              disabled={loading || !task.trim()}
            >
              Submit
            </button>
          </div>
        </form>
      </div>
    </MyTask.Provider>
  );
};
